
var lastTargetOffset = 0.0;

function editorScrollingElement() {
	return tinymce.editors.RSHTMLContent.getDoc().scrollingElement;
}

function setEditorScrollTop(offset) {
	editorScrollingElement().scrollTop = offset;
}

// Utility method for scrolling to line with animation - scroller style
// matches the values from NSScroller.preferredScrollerStyle, and is used to
// dictate whether we hide the scrollbar while scrolling or not.
function scrollToVerticalOffset(offset, scrollerStyle = 0, shouldAnimate = true, duration = 250) {
	// Avoid re-entrant requests to target the same offset
	if (offset == lastTargetOffset) {
		return;
	}
	lastTargetOffset = offset;

	let editorBody = tinymce.editors.RSHTMLContent.getDoc().body;

	if (scrollerStyle != 0) {
		// hide the scroller while scrolling
		editorBody.style["overflow"] = "hidden";
	}

	// scrollingElement is available in Safari 9 or later
	if (shouldAnimate) {
		var startTime = Date.now();
		var startY = editorScrollingElement().scrollTop;
		var endY = offset;
		var delta = endY - startY;

		function easeInOutSine(x) {
			return x < 0.5
			  ? (1 - Math.sqrt(1 - Math.pow(2 * x, 2))) / 2
			  : (Math.sqrt(1 - Math.pow(-2 * x + 2, 2)) + 1) / 2;

			return -(Math.cos(Math.PI * x) - 1) / 2;
		}

		function animate() {
			var now = Date.now();
			var progress = Math.abs(now - startTime) / duration;
			let increasing = endY > startY;

			if (progress > 0.0) {
				var mappedProgress = easeInOutSine(progress);
				var position = startY + (mappedProgress * delta);

				if ((increasing && (position > endY)) || (!increasing && (position < endY))) {
					position = endY; progress = 1.0;
				}
				setEditorScrollTop(position);
			}

			if (progress < 1.0) {
				window.requestAnimationFrame(animate);
			}
			else if (scrollerStyle != 0) {
				editorBody.style["overflow"] = "";
			}
		}

		animate();
	}
	else {
		setEditorScrollTop(offset);
		if (scrollerStyle != 0) {
			editorBody.style["overflow"] = "";
		}
	}
}
